// This work is licensed under the Creative Commons Attribution-ShareAlike 4.0
// International Public License.
// Visit https://creativecommons.org/licenses/by-sa/4.0/.
// SPDX-License-Identifier: CC-BY-SA-4.0
//     povr histogram.pov +w900 +h450 +a0.05 +am2 +r3 +j9.133 +p
//     povr histogram.pov +w1800 +h900 +a0.0 +am2 +r4 +j12.133 +p
//  
// Change function just below to look at a histogram of the scalar values
// returned. Approach derived from Histogram() macro in arraystatistics.inc.
// Generally takes a few seconds a 256 buckets and 100k samples. There is
// an upper limit of 10000 buckets/bins - though practically don't expect this
// will be an issue. When looking at patterns and functions, scale with sampling
// region can occasionally see harmonic effects with unfortunate alignments of
// sampling with the pattern / function results. The sampling here is on regular
// a regular grid. Tested sampling at powers of 2^8 to 2^13.
#version unofficial 3.8; // povr
#if (file_exists("version.inc"))
    #include "version.inc"
#end
#if (!defined(Fork_povr))
    #error "This POV-Ray SDL code requires the povr fork."
#end

#declare Fn00 = function { pattern { wrinkles noise_generator 2 scale 1/pi } }

#declare Samples     = int(1e5);
#declare SamplesSqrt = int(sqrt(Samples));
#declare NoOfBins    = 256;          // Tested with pow(2,8) to pow(2,13)
#declare LoVal       = -0.0-(1e-5);  // Usually 0 or function_interval -1
#declare HiVal       = +1.0+(1e-5);

#declare Xl = -1.0; #declare Xh = 1.0;
#declare Yl = -1.0; #declare Yh = 1.0;
#declare Normalize = yes;  // If yes, scales max bucket % to y = 1.

#declare BinWidth = (HiVal - LoVal)/NoOfBins;
#declare Bins     = array[NoOfBins][3];

#declare BinLimitLow = LoVal;
#for (I, 0, NoOfBins - 1)
    #local BinLimitHigh = BinLimitLow + BinWidth;
    #local Bins[I][0] = BinLimitLow;
    #local Bins[I][1] = BinLimitHigh;
    #local Bins[I][2] = 0; // Set count for bin to zero
    #local BinLimitLow = BinLimitHigh;
#end // for

// The division by 10000 is due the image_map being of 2^16 bit grayscale
// format. The format sets the max number of bins too.
#declare Fn01 = function (x,y,z) {
    min(max(0, // Bound within allowed min/max range of bins
    int((Fn00(x*(Xh-Xl)+Xl,y*(Yh-Yl)+Yl,z) - LoVal)/BinWidth) // The bin num
    ),NoOfBins-1) / 10000
}

#declare Pigment00 = pigment {
    image_map {
        function SamplesSqrt, SamplesSqrt { Fn01(x,y,z) }
        map_type 0
      //interpolate 0  // This I think a povr fork only option.
        once
    }
}

#declare Fn02 = function {
    pigment { Pigment00 scale <(Xh-Xl),(Yh-Yl),1> translate <+Xl,+Yl,0>}
}

#for (Y, 0, SamplesSqrt - 1, 1)
    #declare Yy = Y/SamplesSqrt;
    #for (X, 0, SamplesSqrt - 1, 1)
        #declare Xx = X/SamplesSqrt;
        #declare LocalBinNo = Fn02(Xx*(Xh-Xl)+Xl,Yy*(Yh-Yl)+Yl,0).red * 10000;
        #declare Bins[LocalBinNo][2] = Bins[LocalBinNo][2] + 1;
    #end
#end


//--------- Create the scene to render the histogram

#declare MaxBinPct = 0.0;
#declare Union00 = union {
    #for (II,0,NoOfBins - 1,1)
        #declare Pct    = Bins[II][2] / sqr(SamplesSqrt);
        #declare LocalX = Bins[II][0];
        #if (Pct > 0)
            cylinder {
                <LocalX+BinWidth/2.0,0.0,0.0>,<LocalX+BinWidth/2.0,Pct,0>,
                (BinWidth/2.0 + 1e-5)
            }
        #end
        #debug concat(str(Bins[II][0],10,7)," to ",
            str(Bins[II][1],10,7)," --> ",str(Pct,8,6),"%\n")
        #declare MaxBinPct = max(Pct,MaxBinPct);
    #end
    #if (Normalize)
        scale <1,1/MaxBinPct,1>
    #end
}
#debug concat("Max bucket % -> ",str(MaxBinPct,8,6),"\n")

global_settings { assumed_gamma 1 }
#declare Grey05 = srgb <0.05,0.05,0.05>;
background { Grey05 }
#declare VarOrthoMult =
    3.0/max(image_width/image_height,image_height/image_width);
#declare Camera01z = camera {
    orthographic
    location <0,0.5,-2>
    direction z
    right VarOrthoMult*x*max(1,image_width/image_height)
    up VarOrthoMult*y*max(1,image_height/image_width)
}

object { Union00 pigment { rgb <0,1,0> } finish { emission 1 } }
camera { Camera01z }

// Mark origin and +-1
#declare CylY = cylinder {
    -1*y, 0*y, 0.005
    pigment { rgb <0,0.5,1> } finish { emission 1 }
}
object { CylY }
object { CylY translate -1*x }
object { CylY translate +1*x }

